/* 
 * $Id: HousekeeperServiceHttpImpl.java 3029 2010-06-23 10:41:54Z karianna $
 * $URL: https://open.jira.com/svn/IKASAN/tags/ikasan-0.8.3/platform/jar/src/main/java/org/ikasan/platform/service/HousekeeperServiceHttpImpl.java $
 *
 * ====================================================================
 * Ikasan Enterprise Integration Platform
 * 
 * Distributed under the Modified BSD License.
 * Copyright notice: The copyright for this software and a full listing 
 * of individual contributors are as shown in the packaged copyright.txt 
 * file. 
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the ORGANIZATION nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.ikasan.platform.service;

import java.io.IOException;

import org.apache.http.auth.AuthScheme;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.AuthState;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;

import org.apache.http.HttpException;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpRequestInterceptor;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HttpContext;
import org.apache.log4j.Logger;
import org.ikasan.platform.service.HousekeeperService;

/**
 * A HTTP Implementation of the Housekeeper interface
 * 
 * @author Ikasan Development Team
 */
public class HousekeeperServiceHttpImpl implements HousekeeperService {
	
	/** Logger for this class */
	private static Logger logger = Logger
			.getLogger(HousekeeperServiceHttpImpl.class);

	/** The URL to execute the Wiretap Event Housekeeping */
	private String wiretapEventHousekeepingUrl;
	
	/** User name for login */
	private String wiretapEventHousekeepingUserName;
	
    /** Password required for log in */	
	private String wiretapEventHousekeepingPassword;

	/**
	 * Constructor
	 * 
	 * @param wiretapEventHousekeepingUrl
	 *            - wiretapEventHousekeepingUrl to set
	 * @param wiretapEventHousekeepingUserName - userName
	 * @param wiretapEventHousekeepingPassword - password
	 */
	public HousekeeperServiceHttpImpl(String wiretapEventHousekeepingUrl, String wiretapEventHousekeepingUserName,String wiretapEventHousekeepingPassword)
	{	
		if (wiretapEventHousekeepingUrl == null)
		{
			throw new IllegalArgumentException("wiretapEventHouseKeeping Url is null");
		}
		
		if (wiretapEventHousekeepingUserName == null)
		{
			throw new IllegalArgumentException("wiretapEventHouseKeeping User name is null");
		}
		
		if (wiretapEventHousekeepingPassword == null)
		{
			throw new IllegalArgumentException("wiretapEventHouseKeeping Password is null");
		}
		this.wiretapEventHousekeepingUrl = wiretapEventHousekeepingUrl;
		this.wiretapEventHousekeepingUserName = wiretapEventHousekeepingUserName;
		this.wiretapEventHousekeepingPassword = wiretapEventHousekeepingPassword;
	}

	/**
	 * Housekeeps Wiretapped Events
	 * 
	 * TODO Error handling is lazy and captures any exception, this may actually
	 * be OK but needs review.
	 */
	public void housekeepWiretapEvents()
	{

		// Create the client and set the username and password
		DefaultHttpClient httpclient = new DefaultHttpClient();
		
		//User name and password now passed in through constructors 
		Credentials defaultcreds = new UsernamePasswordCredentials(this.wiretapEventHousekeepingUserName, this.wiretapEventHousekeepingPassword);
		
		// TODO Using AuthScope.ANY is not best practice, we can probably
		// Get the correct details from the URL passed in
		httpclient.getCredentialsProvider().setCredentials(AuthScope.ANY,
				defaultcreds);

		// Generate BASIC scheme object and stick it to the local execution context
		BasicHttpContext localcontext = new BasicHttpContext();
		BasicScheme basicAuth = new BasicScheme();
		localcontext.setAttribute("preemptive-auth", basicAuth);

		// We're going to Preemptively Auth here
		httpclient.addRequestInterceptor(new PreemptiveAuth(), 0);

		HttpPost httppost = new HttpPost(this.wiretapEventHousekeepingUrl);
		try
		{
			HttpResponse response = httpclient.execute(httppost, localcontext);
			int statusCode = response.getStatusLine().getStatusCode();
			// TODO Investigate why we get a 302 back, probably because of the
			// authentication moving us?
			if (!(statusCode == HttpStatus.SC_MOVED_TEMPORARILY || statusCode == HttpStatus.SC_OK))
			{
				logger.error("Call failed, Status Code = [" + statusCode + "]");
				throw new HttpException();
			}
			logger.info("housekeepWiretapEvents was called successfully.");
		}
		// Covers ClientProtocolException as well
		catch (IOException e) 
		{
			logger.error("Call to housekeep failed.", e);
		}
		catch (HttpException e) 
		{
			logger.error("Call to housekeep failed.", e);
		}
		finally
		{
			httpclient.getConnectionManager().shutdown();
		}
	}

	/** Helper class to sort out preemptive auth */
	static class PreemptiveAuth implements HttpRequestInterceptor
	{

		public void process(final HttpRequest request, final HttpContext context)
				throws HttpException, IOException 
		{
			AuthState authState = (AuthState) context
					.getAttribute(ClientContext.TARGET_AUTH_STATE);

			// If no auth scheme available yet, try to initialize it preemptively.
			if (authState.getAuthScheme() == null)
			{
				AuthScheme authScheme = (AuthScheme) context
						.getAttribute("preemptive-auth");
				CredentialsProvider credsProvider = (CredentialsProvider) context
						.getAttribute(ClientContext.CREDS_PROVIDER);
				HttpHost targetHost = (HttpHost) context
						.getAttribute(ExecutionContext.HTTP_TARGET_HOST);
				if (authScheme != null)
				{
					Credentials creds = credsProvider
							.getCredentials(new AuthScope(targetHost
									.getHostName(), targetHost.getPort()));
					if (creds == null)
					{
						throw new HttpException(
								"No credentials for preemptive authentication");
					}
					authState.setAuthScheme(authScheme);
					authState.setCredentials(creds);
				}
			}
		}
	}
}
